/// @file
////////////////////////////////////////////////////////////////////////////////////////////////////
///
/// Copyright (C) 2016/17      Christian Lessig, Otto-von-Guericke Universitaet Magdeburg
///
////////////////////////////////////////////////////////////////////////////////////////////////////
///
///  module     : lecture 8
///
///  author     : lessig@isg.cs.ovgu.de
///
///  project    : GPU Programming
///
///  description: CUDA basics
///
////////////////////////////////////////////////////////////////////////////////////////////////////

#include <iostream>
#include <algorithm>

#include "cuda_util.h"
#include <cuda_runtime.h>
#include <cuda.h>

////////////////////////////////////////////////////////////////////////////////////////////////////
//! Non-local means filter in 1D
////////////////////////////////////////////////////////////////////////////////////////////////////
template< int ApronSize >
void
meanRef( float* data, float* means, int n) {

  for( int i = 0; i < n; ++i) {
    means[i] = 0.0;
    for( int k = i-ApronSize; k <= i+ApronSize; ++k) {
      if( k >= 0 && k < n) {
        means[i] += data[k];
      }
    }
    means[i] /= 2*ApronSize+1;
  }
}

////////////////////////////////////////////////////////////////////////////////////////////////////
//! Non-local means filter in 1D
////////////////////////////////////////////////////////////////////////////////////////////////////
template< int ApronSize >
__global__
void
mean( float* /*gdata*/, int /*n*/) {

}

////////////////////////////////////////////////////////////////////////////////////////////////////
// program entry point
////////////////////////////////////////////////////////////////////////////////////////////////////
int
main( int /*argc*/, char** /*argv*/ ) {

  const int n = 32;
  float* data = (float*) malloc( sizeof(float) * n);
  for( unsigned int i = 0; i < n; ++i) {
    data[i] = (float) i;
  }

  // compute reference solution
  float* meansref = (float*) malloc( sizeof(float) * n);
  meanRef<2>( data, meansref, n);

  // check execution environment

  // set the device

  // allocate device memory

  // copy device memory

  // determine thread layout

  // run kernel

  // copy result back to host

  // output
  bool correct = true;
  for( unsigned int i = 0; i < n; ++i) {
    correct &= (data[i] == meansref[i]);
  }
  std::cout << (correct ? "correct" : "incorrect") << std::endl;

  // clean up device memory

  // clean up host memory
  free( meansref);
  free( data);

  return EXIT_SUCCESS;
}
